@@ -1,34 +1,39 @@
-import concurrent.futures
+from collections.abc import Callable
 from collections.abc import Iterable
+from collections.abc import Sequence
+import concurrent.futures
+from contextlib import contextmanager
+from contextlib import redirect_stdout
 import copy
 import functools
 import hashlib
 import importlib
+import importlib.metadata
 import inspect
 import multiprocessing
 import operator
-import pkg_resources
+import os
 import queue
 import subprocess
 import sys
 import threading
-import typing
 import uuid
 import warnings
 
-from audeer.core import tqdm
+from audeer.core.path import path as safe_path
+from audeer.core.tqdm import progress_bar as audeer_progress_bar
 from audeer.core.version import LooseVersion
 
 
-__doctest_skip__ = ['git_repo_tags', 'git_repo_version']
+__doctest_skip__ = ["git_repo_tags", "git_repo_version"]
 
 
 def deprecated(
-        *,
-        removal_version: str,
-        alternative: str = None
-) -> typing.Callable:
-    """Mark code as deprecated.
+    *,
+    removal_version: str,
+    alternative: str = None,
+) -> Callable:
+    r"""Mark code as deprecated.
 
     Provide a `decorator <https://www.python.org/dev/peps/pep-0318/>`_
     to mark functions/classes as deprecated.
@@ -45,12 +50,13 @@ def deprecated(
         removal_version: version the code will be removed
         alternative: alternative code to use
 
-    Example:
-        >>> @deprecated(removal_version='2.0.0')
+    Examples:
+        >>> @audeer.deprecated(removal_version="2.0.0")
         ... def deprecated_function():
         ...     pass
 
     """
+
     def _deprecated(func):
         # functools.wraps preserves the name
         # and docstring of the decorated code:
@@ -58,23 +64,25 @@ def deprecated(
         @functools.wraps(func)
         def new_func(*args, **kwargs):
             message = (
-                f'{func.__name__} is deprecated and will be removed '
-                f'with version {removal_version}.'
+                f"{func.__name__} is deprecated and will be removed "
+                f"with version {removal_version}."
             )
             if alternative is not None:
-                message += f' Use {alternative} instead.'
+                message += f" Use {alternative} instead."
             warnings.warn(message, category=UserWarning, stacklevel=2)
             return func(*args, **kwargs)
+
         return new_func
+
     return _deprecated
 
 
 def deprecated_default_value(
-        *,
-        argument: str,
-        change_in_version: str,
-        new_default_value: typing.Any,
-) -> typing.Callable:
+    *,
+    argument: str,
+    change_in_version: str,
+    new_default_value: object,
+) -> Callable:
     """Mark default value of keyword argument as deprecated.
 
     Provide a `decorator <https://www.python.org/dev/peps/pep-0318/>`_
@@ -88,15 +96,16 @@ def deprecated_default_value(
         change_in_version: version the default value will change
         new_default_value: new default value
 
-    Example:
-        >>> @deprecated_default_value(
-        ...     argument='foo',
-        ...     change_in_version='2.0.0',
-        ...     new_default_value='bar',
+    Examples:
+        >>> @audeer.deprecated_default_value(
+        ...     argument="foo",
+        ...     change_in_version="2.0.0",
+        ...     new_default_value="bar",
         ... )
-        ... def deprecated_function(foo='foo'):
+        ... def deprecated_function(foo="foo"):
         ...     pass
     """
+
     def _deprecated(func):
         # functools.wraps preserves the name
         # and docstring of the decorated code:
@@ -109,21 +118,24 @@ def deprecated_default_value(
                 message = (
                     f"The default of '{argument}' will change from "
                     f"'{default_value}' to '{new_default_value}' "
-                    f'with version {change_in_version}.'
+                    f"with version {change_in_version}."
                 )
                 warnings.warn(message, category=UserWarning, stacklevel=2)
             return func(*args, **kwargs)
+
         return new_func
+
     return _deprecated
 
 
 def deprecated_keyword_argument(
-        *,
-        deprecated_argument: str,
-        removal_version: str,
-        new_argument: str = None,
-        mapping: typing.Callable = None,
-) -> typing.Callable:
+    *,
+    deprecated_argument: str,
+    removal_version: str,
+    new_argument: str = None,
+    mapping: Callable = None,
+    remove_from_kwargs: bool = True,
+) -> Callable:
     r"""Mark keyword argument as deprecated.
 
     Provide a `decorator <https://www.python.org/dev/peps/pep-0318/>`_
@@ -141,17 +153,21 @@ def deprecated_keyword_argument(
         mapping: if the keyword argument is not only renamed,
             but expects also different input values,
             you can map to the new ones with this callable
+        remove_from_kwargs: if ``True``,
+            ``deprecated_argument`` will be removed
+            from ``kwargs`` inside the decorated object
 
-    Example:
-        >>> @deprecated_keyword_argument(
-        ...     deprecated_argument='foo',
-        ...     new_argument='bar',
-        ...     removal_version='2.0.0',
+    Examples:
+        >>> @audeer.deprecated_keyword_argument(
+        ...     deprecated_argument="foo",
+        ...     new_argument="bar",
+        ...     removal_version="2.0.0",
         ... )
         ... def function_with_new_argument(*, bar):
         ...     pass
 
     """
+
     def _deprecated(func):
         # functools.wraps preserves the name
         # and docstring of the decorated code:
@@ -163,7 +179,10 @@ def deprecated_keyword_argument(
                     f"'{deprecated_argument}' argument is deprecated "
                     f"and will be removed with version {removal_version}."
                 )
-                argument_content = kwargs.pop(deprecated_argument)
+                if remove_from_kwargs:
+                    argument_content = kwargs.pop(deprecated_argument)
+                else:
+                    argument_content = kwargs[deprecated_argument]
                 if new_argument is not None:
                     message += f" Use '{new_argument}' instead."
                     if mapping is not None:
@@ -176,13 +195,13 @@ def deprecated_keyword_argument(
                     stacklevel=2,
                 )
             return func(*args, **kwargs)
+
         return new_func
+
     return _deprecated
 
 
-def flatten_list(
-        nested_list: typing.List
-) -> typing.List:
+def flatten_list(nested_list: list) -> list:
     """Flatten an arbitrarily nested list.
 
     Implemented without  recursion to avoid stack overflows.
@@ -194,13 +213,14 @@ def flatten_list(
     Returns:
         flattened list
 
-    Example:
-        >>> flatten_list([1, 2, 3, [4], [], [[[[[[[[[5]]]]]]]]]])
+    Examples:
+        >>> audeer.flatten_list([1, 2, 3, [4], [], [[[[[[[[[5]]]]]]]]]])
         [1, 2, 3, 4, 5]
-        >>> flatten_list([[1, 2], 3])
+        >>> audeer.flatten_list([[1, 2], 3])
         [1, 2, 3]
 
     """
+
     def _flat_generator(nested_list):
         while nested_list:
             sublist = nested_list.pop(0)
@@ -208,6 +228,7 @@ def flatten_list(
                 nested_list = sublist + nested_list
             else:
                 yield sublist
+
     nested_list = copy.deepcopy(nested_list)
     return list(_flat_generator(nested_list))
 
@@ -223,22 +244,23 @@ def freeze_requirements(outfile: str):
         RuntimeError: if running ``pip freeze`` returns an error
 
     """
-    cmd = f'pip freeze > {outfile}'
-    with subprocess.Popen(
-            args=cmd,
-            stdout=subprocess.DEVNULL,
-            stderr=subprocess.PIPE,
-            shell=True,
-    ) as p:
-        _, err = p.communicate()
-        if bool(p.returncode):
-            raise RuntimeError(f'Freezing Python packages failed: {err}')
+    outfile = safe_path(outfile)
+    try:
+        with open(outfile, "w") as fp:
+            subprocess.run(
+                _pip(["freeze"]),
+                stdout=fp,
+                stderr=subprocess.PIPE,
+                check=True,
+            )
+    except (FileNotFoundError, subprocess.CalledProcessError) as err:
+        raise RuntimeError(f"Freezing Python packages failed: {err}")
 
 
 def git_repo_tags(
-        *,
-        v: bool = None,
-) -> typing.List:
+    *,
+    v: bool = None,
+) -> list:
     r"""Get a list of available git tags.
 
     The tags are inferred by executing
@@ -254,29 +276,31 @@ def git_repo_tags(
     Returns:
         list of tags
 
-    Example:
-        >>> git_repo_tags()
+    Examples:
+        .. skip: next
+
+        >>> audeer.git_repo_tags()
         ['v1.0.0', 'v1.1.0', 'v2.0.0']
 
     """
     try:
-        git = ['git', 'tag']
+        git = ["git", "tag"]
         tags = subprocess.check_output(git)
-        tags = tags.decode().strip().split('\n')
-    except Exception:  # pragma: nocover
+        tags = tags.decode().strip().split("\n")
+    except Exception:  # pragma: no cover
         tags = []
     if v is None:
         return tags
     if v:
-        tags = [f'v{t}' if not t.startswith('v') else t for t in tags]
+        tags = [f"v{t}" if not t.startswith("v") else t for t in tags]
     else:
-        tags = [t[1:] if t.startswith('v') else t for t in tags]
+        tags = [t.removeprefix("v") for t in tags]
     return tags
 
 
 def git_repo_version(
-        *,
-        v: bool = True,
+    *,
+    v: bool = True,
 ) -> str:
     r"""Get a version number from current git ref.
 
@@ -292,29 +316,31 @@ def git_repo_version(
     Returns:
         version number
 
-    Example:
-        >>> git_repo_version()
+    Examples:
+        .. skip: next
+
+        >>> audeer.git_repo_version()
         'v1.0.0'
 
     """
     try:
-        git = ['git', 'describe', '--tags', '--always']
+        git = ["git", "describe", "--tags", "--always"]
         version = subprocess.check_output(git)
         version = version.decode().strip()
-    except Exception:  # pragma: nocover
-        version = '<unknown>'
-    if version.startswith('v') and not v:  # pragma: nocover (only local)
+    except Exception:  # pragma: no cover
+        version = "<unknown>"
+    if version.startswith("v") and not v:  # pragma: no cover (only local)
         version = version[1:]
-    elif not version.startswith('v') and v:  # pragma: nocover (only github)
-        version = f'v{version}'
+    elif not version.startswith("v") and v:  # pragma: no cover (only github)
+        version = f"v{version}"
     return version
 
 
 def install_package(
-        name: str,
-        *,
-        version: str = None,
-        silent: bool = False,
+    name: str,
+    *,
+    version: str = None,
+    silent: bool = False,
 ):
     r"""Install a Python package with pip.
 
@@ -350,24 +376,24 @@ def install_package(
     # check for operators, e.g.
     # >=1.0, >1.0, <=1.0, <1.0
     if version is not None:
-        if version.startswith('>='):
+        if version.startswith(">="):
             op = operator.ge
             version = version[2:].strip()
-        elif version.startswith('>'):
+        elif version.startswith(">"):
             op = operator.gt
             version = version[1:].strip()
-        elif version.startswith('<='):
+        elif version.startswith("<="):
             op = operator.le
             version = version[2:].strip()
-        elif version.startswith('<'):
+        elif version.startswith("<"):
             op = operator.lt
             version = version[1:].strip()
 
     # raise error if package is already installed
     # and does not satisfy requested version
     try:
-        current_version = pkg_resources.get_distribution(name).version
-    except pkg_resources.DistributionNotFound:
+        current_version = importlib.metadata.version(name)
+    except importlib.metadata.PackageNotFoundError:
         current_version = None
 
     if current_version is not None:
@@ -390,20 +416,15 @@ def install_package(
         if op == operator.eq:
             # since we do not support ==1.0
             # we have to add it here
-            name = f'{name}=={version}'
+            name = f"{name}=={version}"
         else:
-            name = f'{name}{version}'
+            name = f"{name}{version}"
 
     subprocess.check_call(
-        [
-            sys.executable,
-            '-m',
-            'pip',
-            'install',
-            name,
-        ],
+        _pip(["install", name]),
         stdout=subprocess.DEVNULL if silent else None,
     )
+
     # This function should be called if any modules
     # are created/installed while your program is running
     # to guarantee all finders will notice the new module’s existence.
@@ -412,9 +433,10 @@ def install_package(
 
 
 def is_semantic_version(version: str) -> bool:
-    r"""Check if given string represents a `semantic version`_.
+    r"""Check if given string represents a semantic version.
 
-    Your version has to comply to ``X.Y.Z`` or ``vX.Y.Z``,
+    To be a `semantic version`_
+    your version has to comply to ``X.Y.Z`` or ``vX.Y.Z``,
     where X, Y, Z are all integers.
     Additional version information, like ``beta``
     has to be added using a ``-`` or ``+``,
@@ -428,16 +450,16 @@ def is_semantic_version(version: str) -> bool:
     Returns:
         ``True`` if version is a semantic version
 
-    Example:
-        >>> is_semantic_version('v1')
+    Examples:
+        >>> audeer.is_semantic_version("v1")
         False
-        >>> is_semantic_version('1.2.3-r3')
+        >>> audeer.is_semantic_version("1.2.3-r3")
         True
-        >>> is_semantic_version('v0.7.2-9-g1572b37')
+        >>> audeer.is_semantic_version("v0.7.2-9-g1572b37")
         True
 
     """
-    version_parts = version.split('.')
+    version_parts = version.split(".")
 
     if len(version_parts) < 3:
         return False
@@ -451,15 +473,14 @@ def is_semantic_version(version: str) -> bool:
 
     x, y = version_parts[:2]
     # Ignore starting 'v'
-    if x.startswith('v'):
-        x = x[1:]
+    x = x.removeprefix("v")
 
-    z = '.'.join(version_parts[2:])
+    z = ".".join(version_parts[2:])
     # For Z, '-' and '+' are also allowed as separators,
-    # but you are not allowed to have an additonal '.' before
-    z = z.split('-')[0]
-    z = z.split('+')[0]
-    if len(z.split('.')) > 1:
+    # but you are not allowed to have an additional '.' before
+    z = z.split("-")[0]
+    z = z.split("+")[0]
+    if len(z.split(".")) > 1:
         return False
 
     for v in (x, y, z):
@@ -480,32 +501,52 @@ def is_uid(uid: str) -> bool:
     Returns:
         ``True`` if string is a unique identifier
 
+    Examples:
+        >>> audeer.is_uid("626f68e6-d336-70b9-e753-ed9fad855840")
+        True
+        >>> audeer.is_uid("ad855840")
+        True
+        >>> audeer.is_uid("not a unique identifier")
+        False
+
     """
     if uid is None:
         return False
     if not isinstance(uid, str):
         return False
+    if len(uid) != 8 and len(uid) != 36:
+        return False
+
+    if len(uid) == 8:
+        uid = f"00000000-0000-0000-0000-0000{uid}"
+
+    for pos in [8, 13, 18, 23]:
+        if not uid[pos] == "-":
+            return False
+
     try:
         uuid.UUID(uid, version=1)
     except ValueError:
         return False
+
     return True
 
 
 def run_tasks(
-        task_func: typing.Callable,
-        params: typing.Sequence[
-            typing.Tuple[
-                typing.Sequence[typing.Any],
-                typing.Dict[str, typing.Any],
-            ]
-        ],
-        *,
-        num_workers: int = 1,
-        multiprocessing: bool = False,
-        progress_bar: bool = False,
-        task_description: str = None
-) -> typing.Sequence[typing.Any]:
+    task_func: Callable,
+    params: Sequence[
+        tuple[
+            Sequence[object],
+            dict[str, object],
+        ]
+    ],
+    *,
+    num_workers: int = 1,
+    multiprocessing: bool = False,
+    progress_bar: bool = False,
+    task_description: str = None,
+    maximum_refresh_time: float = None,
+) -> list[object]:
     r"""Run parallel tasks using multprocessing.
 
     .. note:: Result values are returned in order of ``params``.
@@ -526,11 +567,19 @@ def run_tasks(
         progress_bar: show a progress bar
         task_description: task description
             that will be displayed next to progress bar
+        maximum_refresh_time: refresh the progress bar
+            at least every ``maximum_refresh_time`` seconds,
+            using another thread.
+            If ``None``,
+            no refreshing is enforced
 
-    Example:
-        >>> power = lambda x, n: x ** n
+    Returns:
+        list of computed results
+
+    Examples:
+        >>> power = lambda x, n: x**n
         >>> params = [([2, n], {}) for n in range(10)]
-        >>> run_tasks(power, params, num_workers=3)
+        >>> audeer.run_tasks(power, params, num_workers=3)
         [1, 2, 4, 8, 16, 32, 64, 128, 256, 512]
 
     """
@@ -538,49 +587,54 @@ def run_tasks(
     results = [None] * num_tasks
 
     if num_workers == 1:  # sequential
-
-        with tqdm.progress_bar(
+        with audeer_progress_bar(
             params,
             total=len(params),
             desc=task_description,
+            maximum_refresh_time=maximum_refresh_time,
             disable=not progress_bar,
         ) as pbar:
             for index, param in enumerate(pbar):
                 results[index] = task_func(*param[0], **param[1])
 
     else:  # parallel
-
         if multiprocessing:
             executor = concurrent.futures.ProcessPoolExecutor
         else:
             executor = concurrent.futures.ThreadPoolExecutor
         with executor(max_workers=num_workers) as pool:
-            with tqdm.progress_bar(
+            futures = []
+            try:
+                with audeer_progress_bar(
                     total=len(params),
                     desc=task_description,
+                    maximum_refresh_time=maximum_refresh_time,
                     disable=not progress_bar,
-            ) as pbar:
-                futures = []
-                for param in params:
-                    future = pool.submit(task_func, *param[0], **param[1])
-                    future.add_done_callback(lambda p: pbar.update())
-                    futures.append(future)
-                for idx, future in enumerate(futures):
-                    result = future.result()
-                    results[idx] = result
+                ) as pbar:
+                    for param in params:
+                        future = pool.submit(task_func, *param[0], **param[1])
+                        future.add_done_callback(lambda p: pbar.update())
+                        futures.append(future)
+                    for idx, future in enumerate(futures):
+                        result = future.result()
+                        results[idx] = result
+            except (Exception, KeyboardInterrupt):  # pragma: no cover
+                # Ensure all jobs are canceled immediately
+                pool.shutdown(wait=False, cancel_futures=True)
+                raise
 
     return results
 
 
-@deprecated(removal_version='2.0.0', alternative='run_tasks')
+@deprecated(removal_version="2.0.0", alternative="run_tasks")
 def run_worker_threads(
-        task_fun: typing.Callable,
-        params: typing.Sequence[typing.Any] = None,
-        *,
-        num_workers: int = None,
-        progress_bar: bool = False,
-        task_description: str = None
-) -> typing.Sequence[typing.Any]:  # pragma: no cover
+    task_fun: Callable,
+    params: Sequence[object] = None,
+    *,
+    num_workers: int = None,
+    progress_bar: bool = False,
+    task_description: str = None,
+) -> Sequence[object]:  # pragma: no cover
     r"""Run parallel tasks using worker threads.
 
     .. note:: Result values are returned in order of ``params``.
@@ -596,10 +650,10 @@ def run_worker_threads(
         task_description: task description
             that will be displayed next to progress bar
 
-    Example:
-        >>> power = lambda x, n: x ** n
+    Examples:
+        >>> power = lambda x, n: x**n
         >>> params = [(2, n) for n in range(10)]
-        >>> run_worker_threads(power, params, num_workers=3)
+        >>> audeer.run_worker_threads(power, params, num_workers=3)
         [1, 2, 4, 8, 16, 32, 64, 128, 256, 512]
 
     """
@@ -625,13 +679,13 @@ def run_worker_threads(
 
     # num_workers > 1 -> run parallel
     else:
-
         # Create queue, possibly with a progress bar
         if progress_bar:
+
             class QueueWithProgbar(queue.Queue):
                 def __init__(self, num_tasks, maxsize=0):
                     super().__init__(maxsize)
-                    self.pbar = tqdm.progress_bar(
+                    self.pbar = audeer_progress_bar(
                         total=num_tasks,
                         desc=task_description,
                     )
@@ -639,6 +693,7 @@ def run_worker_threads(
                 def task_done(self):
                     super().task_done()
                     self.pbar.update(1)
+
             q = QueueWithProgbar(num_tasks)
         else:
             q = queue.Queue()
@@ -679,8 +734,8 @@ def run_worker_threads(
 
 
 def sort_versions(
-        versions: typing.List[str],
-) -> typing.List:
+    versions: list[str],
+) -> list[str]:
     """Sort version numbers.
 
     If a version starts with ``v``,
@@ -696,14 +751,14 @@ def sort_versions(
         ValueError: if the version does not comply
             with :func:`is_semantic_version`
 
-    Example:
+    Examples:
         >>> vers = [
-        ...     '2.0.0',
-        ...     '2.0.1',
-        ...     'v1.0.0',
-        ...     'v2.0.0-1-gdf29c4a',
+        ...     "2.0.0",
+        ...     "2.0.1",
+        ...     "v1.0.0",
+        ...     "v2.0.0-1-gdf29c4a",
         ... ]
-        >>> sort_versions(vers)
+        >>> audeer.sort_versions(vers)
         ['v1.0.0', '2.0.0', 'v2.0.0-1-gdf29c4a', '2.0.1']
 
     """
@@ -717,14 +772,30 @@ def sort_versions(
             )
 
     def sort_key(value):
-        if value.startswith('v'):
-            value = value[1:]
+        value = value.removeprefix("v")
         return LooseVersion(value)
 
     return sorted(versions, key=sort_key)
 
 
-def to_list(x: typing.Any):
+@contextmanager
+def suppress_stdout():
+    """Suppress stdout output.
+
+    Any code run inside this context manager
+    does not produce stdout output.
+
+    Example:
+        >>> with suppress_stdout():
+        ...     print("Hello")
+
+    """
+    with open(os.devnull, "w") as devnull:
+        with redirect_stdout(devnull):
+            yield
+
+
+def to_list(x: object):
     """Convert to list.
 
     If an iterable is passed,
@@ -737,10 +808,10 @@ def to_list(x: typing.Any):
     Returns:
         input as a list
 
-    Example:
-        >>> to_list('abc')
+    Examples:
+        >>> audeer.to_list("abc")
         ['abc']
-        >>> to_list((1, 2, 3))
+        >>> audeer.to_list((1, 2, 3))
         [1, 2, 3]
 
     """
@@ -751,32 +822,105 @@ def to_list(x: typing.Any):
 
 
 def uid(
-        *,
-        from_string: str = None,
+    *,
+    from_string: str = None,
+    short: bool = False,
 ) -> str:
     r"""Generate unique identifier.
 
+    A unique identifier contains 36 characters
+    with ``-`` at position 9, 14, 19, 24.
+    If ``short`` is ``True``,
+    only the last 8 digits are returned.
+
     Args:
         from_string: create a unique identifier
             by hashing the provided string.
             This will return the same identifier
             for identical strings.
             If ``None`` :func:`uuid.uuid1` is used.
+        short: if ``True`` returns
+            a short unique identifier
+            (last 8 digits)
 
     Returns:
-        unique identifier containing 36 characters
-        with ``-`` at position 9, 14, 19, 24
+        unique identifier
 
-    Example:
-        >>> uid(from_string='example_string')
+    Examples:
+        >>> audeer.uid(from_string="example_string")
         '626f68e6-d336-70b9-e753-ed9fad855840'
+        >>> audeer.uid(from_string="example_string", short=True)
+        'ad855840'
 
     """
     if from_string is None:
         uid = str(uuid.uuid1())
     else:
         uid = hashlib.md5()
-        uid.update(from_string.encode('utf-8'))
+        uid.update(from_string.encode("utf-8"))
         uid = uid.hexdigest()
-        uid = f'{uid[0:8]}-{uid[8:12]}-{uid[12:16]}-{uid[16:20]}-{uid[20:]}'
+        uid = f"{uid[0:8]}-{uid[8:12]}-{uid[12:16]}-{uid[16:20]}-{uid[20:]}"
+
+    if short:
+        uid = uid[-8:]
+
     return uid
+
+
+def unique(sequence: Iterable) -> list:
+    r"""Unique values in its original order.
+
+    This is an alternative to ``list(set(x))``,
+    which does not preserve the original order.
+
+    Args:
+        sequence: sequence of values
+
+    Returns:
+        unique values from ``x`` in order of appearance
+
+    Examples:
+        >>> list(set([2, 2, 1]))
+        [1, 2]
+        >>> audeer.unique([2, 2, 1])
+        [2, 1]
+
+    """
+    # https://stackoverflow.com/a/480227
+    seen = set()
+    seen_add = seen.add
+    return [x for x in sequence if not (x in seen or seen_add(x))]
+
+
+def _pip(arguments: list[str]) -> list[str]:
+    """Pip command in the given virtual environment.
+
+    The virtual environment can be created with Python,
+    virtualenv,
+    or be managed by uv.
+
+    Returns:
+        command to run pip in current virtual environment
+
+    """
+    if _is_uv():
+        return ["uv", "pip"] + arguments  # pragma: no pip cover
+    return [sys.executable, "-m", "pip"] + arguments  # pragma: no uv cover
+
+
+@functools.lru_cache(maxsize=1)
+def _is_uv() -> bool:
+    """Check if current virtual environment is managed by uv.
+
+    The result is cached for the same virtual environment,
+    to avoid unnecessary disk reads.
+
+    """
+    current_virtual_env_path = sys.prefix
+    pyenv_cfg = os.path.join(current_virtual_env_path, "pyvenv.cfg")
+    if os.path.exists(pyenv_cfg):
+        with open(pyenv_cfg) as fp:  # pragma: no pip cover
+            for line in fp:
+                if line.startswith("uv = "):
+                    return True
+    return False  # pragma: no uv cover
